/***************************************************************************
 *   Copyright (C) 2004 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#include "archiveHandle.h"

#include "kdarConfig.h"
#include "kdardeclarations.h"

#include <kdebug.h>
#include <kurl.h>

#include <qdatetime.h>
#include <qdir.h>
#include <qfile.h>
#include <qregexp.h>
#include <qstring.h>

#include <string>

#include <dar/libdar.hpp>

//For getting the hostname:
#include <unistd.h>

archiveHandle::archiveHandle( KURL archiveURL ) :
    m_archivePath(archiveURL.directory(true, false)),
    m_archiveFilename(archiveURL.fileName())
{
    //Check that the archiveHandle is valid
    trimFilename();
    kdDebug() << "archiveHandle::archiveHandle(): archiveURL is " << archiveURL.path().latin1() << endl;
    kdDebug() << "archiveHandle::archiveHandle(): m_archivePath is " << m_archivePath.latin1() << endl;
    kdDebug() << "archiveHandle::archiveHandle(): m_archiveFilename is " << m_archiveFilename.latin1() << endl;
}

archiveHandle::~archiveHandle()
{
}

void archiveHandle::trimFilename()
{
    m_archiveBasename = m_archiveFilename;
    int pos = m_archiveFilename.find(QRegExp("\\.[1-9][0-9]*\\.dar$"));
    if (pos >= 1)
    {
        kdDebug() << "archiveHandle::checkFilename(): I think it's a .dar file!" << endl;
        m_archiveBasename.truncate(pos);
        kdDebug() << "archiveHandle::checkFilename(): the archiveBasename is " << m_archiveBasename.latin1() << endl;
    }
}

QString archiveHandle::getArchivePath() const
{
    return m_archivePath;
}

QString archiveHandle::getArchiveFilename() const
{
    return m_archiveFilename;
}

QString archiveHandle::getArchiveBasename() const
{
    return m_archiveBasename;
}

libdar::path archiveHandle::getArchiveLibdarPath() const
{
    return libdar::path( m_archivePath.latin1() );
}

std::string archiveHandle::getArchiveBasenameString() const
{
    return m_archiveBasename.latin1();
}

long archiveHandle::lastArchiveSliceNumber() const
{
    //This returns the largest slice number.
    long lastSliceNumber = 0;
    QString filterName( getArchiveBasename() + QString( ".*." ) + EXTENSION );
    QDir currentArchiveDirectory( getArchivePath(), filterName );
    QStringList currentArchiveFiles( currentArchiveDirectory.entryList() );
    long num = 0;
    for ( QStringList::Iterator it = currentArchiveFiles.begin(); it != currentArchiveFiles.end(); ++it )
    {
        (*it).replace( QRegExp( "^.*\\.([1-9][0-9]*)\\.dar$" ), "\\1" );
        bool ok;
        num = (*it).toLong( &ok );
        if ( ok )
        {
            if ( lastSliceNumber < num )
            {
                lastSliceNumber = num;
            }
        }
    }
    return lastSliceNumber;
}

QString archiveHandle::parseStringSubstitutions( const QString & rawString, KDarConfig * kdarconfig )
{
    QString parsedString( rawString );
    QRegExp host( "\\%hostname" );
    QRegExp date( "\\%date" );
    QRegExp time( "\\%time" );
    QRegExp year( "\\%year" );
    QRegExp month( "\\%month" );
    QRegExp day( "\\%day" );
    QRegExp hour( "\\%hour" );
    QRegExp minute( "\\%minute" );
    QRegExp second( "\\%second" );
    QRegExp full( "\\%full" );
    QRegExp diff( "\\%diff" );
    QRegExp refName( "\\%refname" );
    QDateTime dateTime = QDateTime::currentDateTime();
    char hostname[1024];
    if ( gethostname( hostname, sizeof( hostname ) ) == 0 )
    {
        QString hostName( hostname );
        parsedString.replace( host, hostName );
    }
    else
    {
        parsedString.replace( host, "localhost" );
    }
    parsedString.replace( date, dateTime.toString( "yyyyMMdd" ) );
    parsedString.replace( time, dateTime.toString( "hh:mm:ss" ) );
    parsedString.replace( year, dateTime.toString( "yyyy" ) );
    parsedString.replace( month, dateTime.toString( "MM" ) );
    parsedString.replace( day, dateTime.toString( "dd" ) );
    parsedString.replace( hour, dateTime.toString( "hh" ) );
    parsedString.replace( minute, dateTime.toString( "mm" ) );
    parsedString.replace( second, dateTime.toString( "ss" ) );
    parsedString.replace( full, "full" );
    parsedString.replace( diff, "diff" );
    QString refArchiveName( "" );
    if ( kdarconfig ) refArchiveName = kdarconfig->differentialBackupArchiveName();
    if ( !refArchiveName.isEmpty() )
    {
        //We just need the basename, not the entire path:
        archiveHandle refArchiveHandle( refArchiveName );
        parsedString.replace( refName, refArchiveHandle.getArchiveBasename() );
    }
    else
    {
        parsedString.replace( refName, "NO-REF-ARCHIVE" );
    }
    return parsedString;
}

