/***************************************************************************
 *   Copyright (C) 2004 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "createArchiveThread.h"

#include "controller.h"
#include "kdardeclarations.h"
#include "archiveHandle.h"

#include "kdarCheckFileSizeEvent.h"
#include "kdarConfig.h"
#include "kdardialog.h"
#include "kdarListViewItem.h"
#include "kdarProgressEvent.h"
#include "kdarSetCursorEvent.h"
#include "kdarstatistics.h"
#include "kdarStatusBarEvent.h"
#include "kdarWarningEvent.h"
#include "openArchiveThread.h"

#include <kcursor.h>
#include <kdebug.h>
#include <klocale.h>

#include <qcheckbox.h>
#include <qdatetime.h>
#include <qfile.h>
#include <qlistview.h>
#include <qregexp.h>
#include <qstring.h>
#include <qstringlist.h>

#include <dar/libdar.hpp>
#include <dar/erreurs.hpp>

#include <string>

extern bool libdarWorking;

extern libdar::archive *theArchive;

createArchiveThread::createArchiveThread( controller * aController, KDarConfig * kdarconfig )
    : KDarThread( aController, kdarconfig )
{}

createArchiveThread::~createArchiveThread()
{}

void createArchiveThread::run()
{
    libdarWorking = true;
    openLogFile();
    setWaitCursor();
    enableCancelButton();
    logMessage( "\nBEGIN CREATE: " + QDateTime::currentDateTime().toString( "ddd dd MMM yyyy hh:mm:ss" ) + " " + m_kdc->newArchiveName() + "\n", true );
    logMessage( "command line: " + m_kdc->buildDarCommandLine( KDar::LIBDAR_CREATE ) + "\n", true );
    statusMessage( "Creating archive...\n", FOREVER );
    kdDebug() << "createArchiveThread::run(): m_kdc->newArchiveName() is " << m_kdc->newArchiveName().latin1() << endl;
    archiveHandle archiveHndl( m_kdc->newArchiveName() );
    libdar::path fs_root( m_kdc->directoryToBackup().latin1() );
    kdDebug() << "The fs_root is " << fs_root.display().c_str() << endl;

    //Run the backup...

    //Get the current time so we can inform the user how long it took
    //to make the backup.
    QDateTime startTime = QDateTime::currentDateTime();
    //Remove any archives of the same path that already exist
    //See dar-libdar_api@lists.sf.net mailing list for discussion of
    //removing files from disk
    int i( 1 );
    QFile oldArchive( m_kdc->newArchiveName() + QString( "." ) + QString::number( i ) + QString( "." ) + EXTENSION );
    while ( oldArchive.exists() )
    {
        kdDebug() << "removing " << oldArchive.name().latin1() << endl;
        oldArchive.remove();
        oldArchive.setName( m_kdc->newArchiveName()+QString( "." ) + QString::number( ++i ) + QString( "." ) + EXTENSION );
    }

    //Start the progressbar timer;
    startProgressBarUpdates();
    //Start the creationCheckFileSizeTimer
//    kdarCheckFileSizeEvent *checkFileSizeEventStart = new kdarCheckFileSizeEvent( kdarCheckFileSizeEvent::STARTTIMER );
//    QApplication::postEvent( m_kdar, checkFileSizeEventStart );

    if ( !m_kdc->differentialBackup() )
    {
        try
        {
            kdDebug() << "createArchiveThread::run(): fs_root: " << fs_root.display().c_str() << endl;
            kdDebug() << "createArchiveThread::run(): archiveHndl.getArchiveLibdarPath(): " << archiveHndl.getArchiveLibdarPath().display().c_str() << endl;
            kdDebug() << "createArchiveThread::run(): theArchive: N/A" << endl;
            kdDebug() << "createArchiveThread::run(): archiveHndl.getArchiveBasename(): " << archiveHndl.getArchiveBasename().latin1() << endl;
            kdDebug() << "createArchiveThread::run(): m_kdc->allowOverwrite(): " << m_kdc->allowOverwrite() << endl;
            kdDebug() << "createArchiveThread::run(): m_kdc->warnOnOverwrite(): " << m_kdc->warnOnOverwrite() << endl;
            kdDebug() << "createArchiveThread::run(): m_kdc->verbose(): " << m_kdc->verbose() << endl;
            kdDebug() << "createArchiveThread::run(): m_kdc->pauseBetweenSlices(): " << m_kdc->pauseBetweenSlices() << endl;
            kdDebug() << "createArchiveThread::run(): m_kdc->keepPruned(): " << m_kdc->keepPruned() << endl;
            kdDebug() << "createArchiveThread::run(): m_kdc->compressionAlgorithm: " << (int) m_kdc->compressionAlgorithm() << endl;
            kdDebug() << "createArchiveThread::run(): m_kdc->compressionLevel(): " << m_kdc->compressionLevel() << endl;
            kdDebug() << "createArchiveThread::run(): m_kdc->libdarSliceSize(): " << libdar::deci(m_kdc->libdarSliceSize()).human().c_str() << endl;
            kdDebug() << "createArchiveThread::run(): m_kdc->libdarFirstSliceSize(): " << libdar::deci(m_kdc->libdarFirstSliceSize()).human().c_str() << endl;
            kdDebug() << "createArchiveThread::run(): m_kdc->systemEA(): " << m_kdc->systemEA() << endl;
            kdDebug() << "createArchiveThread::run(): m_kdc->userEA(): " << m_kdc->userEA() << endl;
            kdDebug() << "createArchiveThread::run(): m_kdc->executeCommand(): " << m_kdc->executeCommand().latin1() << endl;
            kdDebug() << "createArchiveThread::run(): m_kdc->passwordString(): " << m_kdc->passwordString().latin1() << endl;
            kdDebug() << "createArchiveThread::run(): m_kdc->minimumCompressionSizeInt(): " << m_kdc->minimumCompressionSizeInt() << endl;
            kdDebug() << "createArchiveThread::run(): m_kdc->ignoreDump(): " << m_kdc->ignoreDump() << endl;
            kdDebug() << "createArchiveThread::run(): m_kdc->ignoreID(): " << m_kdc->ignoreID() << endl;
            kdDebug() << "createArchiveThread::run(): m_kdc->hourShiftInt(): " << libdar::deci(m_kdc->hourShiftInt()).human().c_str() << endl;
            kdDebug() << "createArchiveThread::run(): m_kdc->:dryRun()" << m_kdc->dryRun() << endl;

            KDarStatistics ks=op_create(fs_root, archiveHndl.getArchiveLibdarPath(), NULL, m_kdc->libdarCreateFileMask(), m_kdc->libdarCreateDirectoryMask(), archiveHndl.getArchiveBasename().latin1(), EXTENSION, m_kdc->allowOverwrite(), m_kdc->warnOnOverwrite(), m_kdc->verbose(), m_kdc->pauseBetweenSlices(), m_kdc->keepPruned(), (libdar::compression) m_kdc->compressionAlgorithm(), (libdar::U_I) m_kdc->compressionLevel(), m_kdc->libdarSliceSize(), m_kdc->libdarFirstSliceSize(), m_kdc->systemEA(), m_kdc->userEA(), m_kdc->executeCommand().latin1(), libdar::crypto_none, m_kdc->passwordString().latin1(), m_kdc->libdarCreateCompressionMask(), (libdar::U_I) m_kdc->minimumCompressionSizeInt(), m_kdc->ignoreDump(), m_kdc->ignoreID(), (libdar::infinint) m_kdc->hourShiftInt(), m_kdc->dryRun() );
            statusMessage( "Creating archive...done.\n" );
            logMessage( ks.createStatistics( (KDar::logLevel) m_kdc->logLevel() ) );
            //Post a messageEvent showing info:
            logMessage( slicesWritten() );
            logMessage( timeDifference( startTime, QDateTime::currentDateTime() ) );
        }
        catch(libdar::Egeneric &e)
        {
            libdar::user_interaction_warning(e.get_message());
        }
    }
    else
    {
        m_controller->waitForThread( controller::OPEN_THREAD );
        if ( theArchive )
        {
            try
            {
                KDarStatistics ks=op_create(fs_root, archiveHndl.getArchiveLibdarPath(), theArchive, m_kdc->libdarCreateFileMask(), m_kdc->libdarCreateDirectoryMask(), archiveHndl.getArchiveBasename().latin1(), EXTENSION, m_kdc->allowOverwrite(), m_kdc->warnOnOverwrite(), m_kdc->verbose(), m_kdc->pauseBetweenSlices(), m_kdc->keepPruned(), (libdar::compression) m_kdc->compressionAlgorithm(), (libdar::U_I) m_kdc->compressionLevel(), m_kdc->libdarSliceSize(), m_kdc->libdarFirstSliceSize(), m_kdc->systemEA(), m_kdc->userEA(), m_kdc->executeCommand().latin1(), libdar::crypto_none, m_kdc->passwordString().latin1(), m_kdc->libdarCreateCompressionMask(), (libdar::U_I) m_kdc->minimumCompressionSizeInt(), m_kdc->ignoreDump(), m_kdc->ignoreID(), (libdar::infinint) m_kdc->hourShiftInt(), m_kdc->dryRun() );
                statusMessage( "Creating archive...done.\n" );
                logMessage( ks.createStatistics( (KDar::logLevel) m_kdc->logLevel() ) );
                //Post a messageEvent showing info:
                logMessage( slicesWritten() );
                logMessage( timeDifference( startTime, QDateTime::currentDateTime() ) );
            }
            catch(libdar::Egeneric &e)
            {
                libdar::user_interaction_warning(e.get_message());
            }
            catch( ... )
            {
                libdar::user_interaction_warning( "createArchiveThread::run(): Caught exception during differential creation!\n" );
            }
        }
        else
        {
            statusMessage( "Could not open reference archive: creation aborted.\n" );
        }
    }
    ///TODO: Report some statistics...

    setNormalCursor();
    //Stop the progressbar timer;
    stopProgressBarUpdates();
    //Disable the cancel button command.
    disableCancelButton();
    //Stop checking the archive slice sizes.
//    kdarCheckFileSizeEvent *checkFileSizeEventStop = new kdarCheckFileSizeEvent( kdarCheckFileSizeEvent::STOPTIMER );
//    QApplication::postEvent( m_kdar, checkFileSizeEventStop );
    //This message has to be posted from the KDarThread destructor so that
    //the thread has had a chance to finish all of its logging output.
    m_endMessage = "END CREATE: " + QDateTime::currentDateTime().toString( "ddd dd MMM yyyy hh:mm:ss" ) + " " + m_kdc->newArchiveName() + "\n";
    libdarWorking = false;

    deleteMe();
}

QString createArchiveThread::slicesWritten()
{
    //Get the last slice number:
    archiveHandle name( m_kdc->newArchiveName() );
    long lastSliceNumber = name.lastArchiveSliceNumber();
    //The info string: contains archive name, slices created and time elapsed.
    QString slices( name.getArchiveBasename() + ": " );
    slices.append( i18n( "wrote %n slice.\n", "wrote %n slices.\n", lastSliceNumber ) );
    return slices;
}
