/***************************************************************************
 *   Copyright (C) 2004 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "getUserResponseThread.h"

#include "controller.h"
#include "kdarProgressEvent.h"
#include "kdarStatusBarEvent.h"
#include "kdarTarListingEvent.h"
#include "kdarUserInteractionEvent.h"
#include "kdarWarningEvent.h"

#include <kdebug.h>
#include <klocale.h>

#include <qapp.h>
#include <qregexp.h>
#include <qsemaphore.h>
#include <qthread.h>

extern kdarListViewItem *currentDirectory;
extern bool gotUserResponse;
extern QSemaphore *mySemaphore;

//We have to declare this outside the getUserResponseThread:: scope
//since libdar does not provide us with a context value for the callbacks.
controller * m_controller;

getUserResponseThread::getUserResponseThread( controller * aController )
        : QThread( 0 )
{
    m_controller = aController;
}

getUserResponseThread::~getUserResponseThread()
{
}

void getUserResponseThread::run()
{
}

// The user message callback function required by libdar
void getUserResponseThread::interaction_warning_display(const std::string &message)
{
    //We have to post a custom event here to display the message because we're not the GUI thread.
    QString userMessage = message.c_str();
    //Are we adding a file to the archive? Just post a statusBarEvent
    //because large archive creation slows the GUI responsiveness to a halt!

    //This might be better implemented with a QTimer that checks the number
    //of entries in the message window and deletes 50% of them if
    //the number is larger than some threshold, say 1000 lines...
    //Also, need to implement logging the messages to a logfile.

    //Implementing a progress bar that shows the percentage of total files
    //that have been processed, as a function of the total number of files
    //to be processed.

    bool sendToStatusBar = false;
    bool increaseProgressBar = false;

    QRegExp addingRegExp = QRegExp( i18n( "libdar create message", "^Adding file to archive: " ) );
    QRegExp diffOKRegExp = QRegExp( i18n( "libdar diff message", "^OK   " ) );
    QRegExp testOKRegExp = QRegExp( i18n( "libdar test message", "^OK  <ROOT>" ) );
    QRegExp restoringRegExp = QRegExp( i18n( "libdar restore message", "^restoring file " ) );
//    QRegExp extractingRegExp = QRegExp( "^Extracting contents of the archive..." );
    if ( userMessage.contains( addingRegExp ) )
    {
        userMessage.replace( addingRegExp, i18n( "statusbar replacement for creation message", "Adding " ) );
        sendToStatusBar = true;
        increaseProgressBar = true;
    }
    else if ( userMessage.contains( diffOKRegExp ) || userMessage.contains( testOKRegExp ) )
    {
        sendToStatusBar = true;
        increaseProgressBar = true;
    }
    else if ( userMessage.contains( restoringRegExp ) )
    {
        userMessage.replace( restoringRegExp, i18n( "statubar replacement for restore message", "Restoring " ) );
        sendToStatusBar = true;
        increaseProgressBar = true;
    }
/*    else if ( userMessage.contains( extractingRegExp ) )
    {
        //Do nothing
        //KDar displays this information.
    }
*/
    if ( sendToStatusBar )
    {
        //Post the statusBar event
        kdarStatusBarEvent *statusEvent = new kdarStatusBarEvent( userMessage, FOREVER );
        QApplication::postEvent( m_controller, statusEvent );
    }
    else
    {
        kdarWarningEvent *warnEvent = new kdarWarningEvent( userMessage );
        QApplication::postEvent( m_controller, warnEvent );
    }
    if ( increaseProgressBar )
    {
        //increase the progress counter
        kdarProgressEvent *addEvent = new kdarProgressEvent( kdarProgressEvent::PROGRESS_INCREASE_ADDED );
        QApplication::postEvent( m_controller, addEvent );
    }
}

//This is the tar listing callback used by libdar
void getUserResponseThread::tarListing( const std::string & flag,
                                        const std::string & perm,
                                        const std::string & uid,
                                        const std::string & gid,
                                        const std::string & size,
                                        const std::string & date,
                                        const std::string & filename)
{
    //We have to post a custom event here to display the listing because we're not the GUI thread.
    QString qflag = flag.c_str();
    QString qperm = perm.c_str();
    QString quid = uid.c_str();
    QString qgid = gid.c_str();
    QString qsize = size.c_str();
    QString qdate = date.c_str();
    QString qfilename = filename.c_str();
    kdDebug() << "getUserResponseThread::tarListing() reached: posting a tarListing Event." << endl;
    //create and post a new tarListingEvent
    kdarTarListingEvent *tarListingEvent = new kdarTarListingEvent( currentDirectory, qflag, qperm, quid, qgid, qsize, qdate, qfilename );
    QApplication::postEvent( m_controller, tarListingEvent );
}

//This is the user interaction callback function required by libdar
bool getUserResponseThread::interaction_pause(const std::string &message)
{
    //Don't access the GUI directly here because we're not the GUI thread.
    QString userQuestion = message.c_str();
    kdarUserInteractionEvent *questionEvent = new kdarUserInteractionEvent( userQuestion );
    //reset the gotUserResponse boolean:
    gotUserResponse = false;
    //post the questionEvent
    QApplication::postEvent( m_controller, questionEvent );
    kdDebug() << "interaction_pause: going to sleep..."  << endl;
    (*mySemaphore)++;
    kdDebug() << "interaction_pause: waking up!" << endl;
    kdDebug() << "interaction_pause: User answered " << gotUserResponse << "." << endl;
    return gotUserResponse;
}
