/***************************************************************************
 *   Copyright (C) 2004 by Johnathan Burchill                              *
 *   jkerrb@shaw.ca                                                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "isolateArchiveThread.h"

#include "kdardeclarations.h"
#include "archiveHandle.h"
#include "kdarConfig.h"
#include "kdardeletethreadevent.h"
#include "kdarSetCursorEvent.h"
#include "kdarStatusBarEvent.h"

#include <kcursor.h>

#include <dar/libdar.hpp>
#include <string>

extern bool libdarWorking;
extern libdar::archive *theArchive;

isolateArchiveThread::isolateArchiveThread( controller * aController, KDarConfig * kdarconfig )
    : KDarThread( aController, kdarconfig )
{}

isolateArchiveThread::~isolateArchiveThread()
{}

void isolateArchiveThread::run()
{
    libdarWorking = true;
    setWaitCursor();
    enableCancelButton();
    openLogFile();
    logMessage( "\nBEGIN ISOLATE: " + QDateTime::currentDateTime().toString( "ddd dd MMM yyyy hh:mm:ss" ) + " " + m_kdc->archiveName() + "\n", true );
    logMessage( "command line: " + m_kdc->buildDarCommandLine( KDar::LIBDAR_ISOLATE ) + "\n", true );
    statusMessage( "Isolating archive...\n", FOREVER );

    std::string filename;
    QString isoArchiveName = m_kdc->isolationArchiveName();

    if (isoArchiveName[0] == '/')
    {
        archiveHandle archiveHndl( m_kdc->isolationArchiveName() );
        //isolate the archive
        try
        {
            libdar::op_isolate( archiveHndl.getArchiveLibdarPath(),
                                theArchive,
                                archiveHndl.getArchiveBasename().latin1(),
                                EXTENSION,
                                m_kdc->allowOverwrite(),
                                m_kdc->warnOnOverwrite(),
                                m_kdc->verbose(),
                                m_kdc->pauseBetweenSlices(),
                                (libdar::compression) m_kdc->compressionAlgorithm(),
                                (libdar::U_I) m_kdc->compressionLevel(),
                                m_kdc->libdarSliceSize(),
                                m_kdc->libdarFirstSliceSize(),
                                m_kdc->executeCommand().latin1(),
                                libdar::crypto_none, m_kdc->passwordString().latin1() );
            //Inform user
            statusMessage( "Isolating archive...done.\n" );
        }
        catch( libdar::Egeneric &e )
        {
            libdar::user_interaction_warning( e.get_message() );
            statusMessage( "Error: could not isolate archive.\n" );
        }
    }
    setNormalCursor();
    disableCancelButton();
    m_endMessage = "END ISOLATE: " + QDateTime::currentDateTime().toString( "ddd dd MMM yyyy hh:mm:ss" ) + " " + m_kdc->archiveName() + "\n";
    libdarWorking = false;
    deleteMe();
}

