/***************************************************************************
 *   Copyright (C) 2004 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef _KDARCONTROLLER_H_
#define _KDARCONTROLLER_H_

#include "controller.h"

#include <qfile.h>
#include "qobject.h"
#include <qtextstream.h>

class KDarConfig;
class kdarView;

class createArchiveThread;
class diffArchiveThread;
class getUserResponseThread;
class isolateArchiveThread;
class listArchiveThread;
class openArchiveThread;
class populateTreeArchiveThread;
class restoreArchiveThread;
class testArchiveThread;

class QEvent;

/**
 * This is the KDar controller class. It controls the libdar operations
 * based on user inputs, and is responsible for passing results on to the main
 * view.
 *
 * @short KDar Controller
 * @author Johnathan Burchill, <jkerrb@users.sourceforge.net>
 * @version VERSION
 */

class KDarController : public controller
{
    Q_OBJECT

public:

    KDarController();
    ~KDarController();

    /**
     *  setParent sets the parent widget
     */
    void setParent( void* );
    /**
     *  closeArchive is a controller method to close an archive
     *  in anticipation of opening a new archive.
     */
    void closeArchive();
    /**
     * Closes the logFile.
     */
    void closeLogFile();
    /**
     *  createArchive is a controller method to create an archive
     *  from filesystem data.
     *
     * @param updateListView whether to update the listView with the
     * reference archive when doing a differential backup. Defaults to true.
     */
    void createArchive( KDarConfig *, bool updateListView = true );

    /**
     *  diffArchive is a controller method to compare an archive
     *  against the filesystem.
     */
    void diffArchive( KDarConfig * );

    /**
     *  isolateArchive is a controller method to isolate an archive
     *  catalog into its own file.
     */
    void isolateArchive( KDarConfig * );

    /**
     *  listArchive is a controller method to initiate listing the
     *  contents of an archive in the browser window (KListView).
     */
    void listArchive( KDarConfig *, bool loadArchive = true );

    /**
     *  openArchive is a controller method to open an archive.
     */
    void openArchive( KDarConfig *, bool updateListView = true );
    /**
     * Opens the logFile for appending.
     */
    void openLogFile( const QString & logFile );
    /**
     *  populateArchiveTree is a controller method to list the contents
     *  of a subdirectory in an archive. It needs to receive a QListViewItem
     */
    void populateArchiveTree( KDarConfig *, QListViewItem * );
    /**
     *  restoreArchive is a controller method to restore the contents
     *  of an archive to the filesystem.
     */
    void restoreArchive( KDarConfig * );
    /**
     *  testArchive is a controller method to test an archive.
     */
    void testArchive( KDarConfig * );
    /**
     *  waitForThread checks to see if the given thread type
     *  is running, and waits for it to finish.
     */
    void waitForThread( controller::threadType );
    /**
     * Writes a message to the logfile.
     */
    void writeLogMessage( QString & message );
    /**
     *  customEvent deals with custom events
     */
    void customEvent( QCustomEvent *e );

private:

    createArchiveThread *m_createThread;
    diffArchiveThread *m_diffThread;
    isolateArchiveThread *m_isolateThread;
    listArchiveThread *m_listThread;
    openArchiveThread *m_openThread;
    populateTreeArchiveThread *m_populateTreeThread;
    restoreArchiveThread *m_restoreThread;
    testArchiveThread *m_testThread;
    getUserResponseThread *m_userResponseThread;

    kdarView *m_parent;

    QFile m_logFile;
    QTextStream m_logStream;

};

#endif // _KDARCONTROLLER_H_
