/*
 * Copyright (C) 2003, 2004 Johnathan Burchill
 * <jkerrb@users.sourceforge.net>
 */

#include "kdarview.h"

#include "archiveHandle.h"
#include "controller.h"
#include "kdarConfig.h"
#include "kdar.h"
#include "kdardeclarations.h"
#include "kdarEnums.h"
#include "kdarListViewItem.h"
#include "kdarArchiveBrowserUpdatesEvent.h"
#include "kdarCancelButtonEvent.h"
#include "kdarCheckFileSizeEvent.h"
#include "kdarPostListViewItemEvent.h"
#include "kdarProgressEvent.h"
#include "kdarSetCursorEvent.h"
#include "kdarStatusBarEvent.h"
#include "kdarTarListingEvent.h"
#include "kdarUserInteractionEvent.h"
#include "kdarWarningEvent.h"

#include "newarchivestoragedirdialog.h"

#include <kaction.h>
#include <kconfig.h>
#include <kcursor.h>
#include <kdebug.h>
#include <kdialogbase.h>
#include <kfiledialog.h>
#include <kfileitem.h>
#include <kiconloader.h>
#include <kio/job.h>
#include <klibloader.h>
#include <klineedit.h>
#include <klistview.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kmimetype.h>
#include <kprinter.h>
#include <kprogress.h>
#include <krun.h>
#include <kstatusbar.h>
#include <ktextbrowser.h>
#include <ktrader.h>
#include <kurl.h>
#include <kurlrequester.h>
#include <kurlpixmapprovider.h>
#include <kparts/componentfactory.h>

#include <qcheckbox.h>
#include <qdragobject.h>
#include <qevent.h>
#include <qfile.h>
#include <qframe.h>
#include <qlayout.h>
#include <qpaintdevicemetrics.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qregexp.h>
#include <qsemaphore.h>
#include <qsimplerichtext.h>
#include <qstring.h>
#include <qstylesheet.h>
#include <qtimer.h>
#include <qvalidator.h>
#include <qvbox.h>
#include <qwaitcondition.h>

#include <iostream>

#include <dar/libdar.hpp>

//auto_ptr
#include <memory>

extern bool gotUserResponse;
extern libdar::archive *theArchive;

//archiveItem is the basename of the archive
kdarListViewItem * archiveItem;


QSemaphore *mySemaphore;

kdarView::kdarView( controller *aController, QWidget *parent )
    : DCOPObject( "kdarIface" ),
      KDARDlg( parent ),
      createDlg( new kdarCreateDialog( this ) ),
      createDlgWizard( new kdarCreateArchiveWizard( this ) ),
      extractDlg( new kdarExtractDialog ),
      m_controller( aController ),
      kdarPixmapProvider( new KURLPixmapProvider ),
      fileSizeTimer( 0 ),
      progressTimer( 0 ),
      progressBar( 0 ),
      logDialog( 0 )
{
    kdDebug() << "kdarView::kdarView(): entering constructor" << endl;
    //Configure some settings for the archive browser:
    kdDebug() << "kdarView::kdarView(): configuring archive browser" << endl;
    archiveFileView->setColumnWidthMode( 0, QListView::Manual );
    archiveFileView->setColumnWidthMode( 6, QListView::Maximum );
    archiveFileView->setSorting( 1 );

    //Set right alignment on all columns except the first
    kdDebug() << "kdarView::kdarView(): set right alignment on browser" << endl;
    int numCols = archiveFileView->columns();
    for( int i = 1; i < numCols; ++i )
    {
        archiveFileView->setColumnAlignment( i, Qt::AlignRight );
    }
    //The browser won't accept drops. What is wrong?
    kdDebug() << "kdarView::kdarView(): dnd-setup" << endl;
    setAcceptDrops( true );
    archiveFileView->setAcceptDrops( true );
    archiveFileView->setDragEnabled( true );

    //Set the controller's parent to this
    kdDebug() << "kdarView::kdarView(): m_controller->setParent()" << endl;
    m_controller->setParent( this );
    //Instantiate the KDarConfig object only AFTER the callback functions
    //have been set. Perhaps the callbacks should be set in m_controller?
    kdDebug() << "kdarView::kdarView(): new KDarConfig( this )" << endl;
    m_kdarconfig = new KDarConfig( this );

    //Now copy the m_kdarconfig object to the create and restore config
    //objects
    kdDebug() << "kdarView::kdarView(): copying m_kdarconfig to m_createConfig" << endl;
    m_createConfig = new KDarConfig( this, 0, m_kdarconfig, false );
    kdDebug() << "kdarView::kdarView(): copying m_kdarconfig to m_restoreConfig" << endl;
    m_restoreConfig = new KDarConfig( this, 0, m_kdarconfig, false );

    //Set the createConfig object to display only the relevant options:
    kdDebug() << "kdarView::kdarView(): m_createConfig->setupCreateConfig()" << endl;
    m_createConfig->setupCreateConfig();
    //Set the restoreConfig object to display only the relevant options:
    kdDebug() << "kdarView::kdarView(): m_restoreConfig->setupRestoreConfig()" << endl;
    m_restoreConfig->setupRestoreConfig();

    //Set the createDlg's KDarConfig pointer:
    kdDebug() << "kdarView::kdarView(): createDlg->setKDarConfig()" << endl;
    createDlg->setKDarConfig( m_createConfig );
    //Set the createWizard's KDarConfig pointer:
    kdDebug() << "kdarView::kdarView(): createDlgWizard->setKDarConfig()" << endl;
    createDlgWizard->setKDarConfig( m_createConfig );

    //Create dialog connections
    //Connect the createDlg "configure" clicked() signal to its slot

    kdDebug() << "kdarView::kdarView(): QSemaphore craziness" << endl;
    mySemaphore = new QSemaphore(1);
    (*mySemaphore)++;

    //set up an integer validator on the slicesize lineEdits
    kdDebug() << "kdarView::kdarView(): integer validator for lineEdits" << endl;
    QRegExp rx("[1-9][0-9]*");
    QRegExpValidator * restrictToWholeNumbers = new QRegExpValidator(rx, this);
    m_kdarconfig->sliceSizeLineEdit->setValidator( restrictToWholeNumbers );
    m_kdarconfig->firstSliceSizeLineEdit->setValidator( restrictToWholeNumbers );

    logDialog = new KDialogBase( KDialogBase::Plain, i18n("caption for logDialog", "Message Log" ), 0/*KDialogBase::Close*/, KDialogBase::Default/*KDialogBase::Close*/, this, 0, false );
    QFrame *page = logDialog->plainPage();
    QVBoxLayout *logLayout = new QVBoxLayout( page, 0, logDialog->spacingHint() );
    logPart = KParts::ComponentFactory::createPartInstanceFromQuery<KParts::ReadOnlyPart>( "text/x-log", QString::null, page, 0, page, 0 );
    if ( logPart )
    {
        logPart->widget()->setMinimumWidth( logDialog->fontMetrics().maxWidth()*5 );
        logPart->widget()->setMinimumHeight( logDialog->fontMetrics().height()*5 );
        logLayout->addWidget( logPart->widget() );
//        part->widget()->show();
    }
    int logWidth = logDialog->fontMetrics().maxWidth()*20;
    int logHeight = logDialog->fontMetrics().height()*15;
    logDialog->resize( logWidth, logHeight );
//    logDialog->show();
}

kdarView::~kdarView()
{
    saveProperties();
    if ( m_kdarconfig ) delete m_kdarconfig;
    if ( createDlg ) delete createDlg;
    if ( createDlgWizard ) delete createDlgWizard;
    if ( extractDlg ) delete extractDlg;
}

void kdarView::dropEvent(QDropEvent *event)
{
    kdDebug() << "kdarView::dropEvent() reached." << endl;

    // this is a very simplistic implementation of a drop event.  we
    // will only accept a dropped URL.  the Qt dnd code can do *much*
    // much more, so please read the docs there
    QStrList uri;

    // see if we can decode a URI.. if not, just ignore it
    if ( QUriDrag::decode( event, uri ) )
    {
        // okay, we have a URI.. process it
        QString url, target;
        url = uri.first();
        KURL theURL = KURL::fromPathOrURL( url );
        // okay, somehow list this archive? or just select it?
        if ( theURL.isLocalFile() )
        {
            kdDebug() << "kdarView::dropEvent(): Local file!" << endl;
            if ( url.find(QRegExp("\\.[1-9][0-9]*\\.dar") ) >= 1)
            {
                kdDebug() << "kdarView::dropEvent(): I think it's a disk archive file" << endl;
                kdDebug() << "kdarView::dropEvent(): FIXME: slotArchiveChoose( theURL.path() );" << endl;
                //slotArchiveChoose( theURL.path() );
            }
            else
            {
                kdDebug() << "kdarView::dropEvent(): Not a disk archive file" << endl;
            }
        }
        kdDebug() << "kdarView::dropEvent(): You dropped..." << url.latin1() << endl;
        kdDebug() << "kdarView::dropEvent(): The path is... " << theURL.path().latin1() << endl;
    }
}

bool kdarView::event( QEvent * e )
{
    //Need a waitcondition for user interaction
    QWaitCondition userResponse;
    if ( e->type() >= QEvent::User )
    {
        if ( e->type() == ( QEvent::Type ) KDar::WARNING )
        {
            kdarWarningEvent* we = ( kdarWarningEvent* )e;
            QString message = we->message();
            archiveTextBrowser->append( message );
            //Log this message if log level is KDar::LOG_FULL
            if ( m_kdarconfig->logLevel() == KDar::LOG_FULL && !message.isEmpty() )
            {
                m_controller->writeLogMessage( message );
            }
            return true;
        }
        if ( e->type() == (QEvent::Type) KDar::USERINTERACTION )
        {
            bool fileSizeTimerRunning = false;
            if ( fileSizeTimer ) fileSizeTimerRunning = fileSizeTimer->isActive();
            if ( fileSizeTimerRunning ) fileSizeTimer->stop();
            //kdDebug() << "kdar::event: It is a kdarUserInteractionEvent. Asking the question..." << endl;
            kdarUserInteractionEvent* uie = (kdarUserInteractionEvent*)e;
            QString question( uie->question() );
            int ret = KMessageBox::warningYesNo(0, question );
            gotUserResponse = (ret == KMessageBox::Yes);
            //kdDebug() << "kdar::event: User answered " << gotUserResponse<< ". Wake up the interaction_pause thread." << endl;
            (*mySemaphore)--;
            //kdDebug() << "kdar::event: sent wake-up signal." << endl;
            //Log this message if log level is KDar::LOG_FULL
            if ( m_kdarconfig->logLevel() == KDar::LOG_FULL && !question.isEmpty() )
            {
                QString answer( i18n( "Answer to user interaction question", "NO." ) );
                if ( gotUserResponse ) answer = i18n( "Answer to user interaction question", "YES." );
                question += QString( "..." ) + answer;
                m_controller->writeLogMessage( question );
            }

            if ( fileSizeTimerRunning ) fileSizeTimer->start( CHECKINTERVAL );
            return true;
        }
        if ( e->type() == (QEvent::Type) KDar::TARLISTING )  // It must be a kdarTarListingEvent
        {
            //kdDebug() << "kdar::event: It is a kdarTarListingEvent...updating listview." << endl;
            kdarTarListingEvent* te = (kdarTarListingEvent*)e;
            kdarListViewItem *directory = te->directory();
            QString filename = te->filename();
            QString perm = te->perm();
            QString uid = te->uid();
            QString gid = te->gid();
            QString size = te->size();
            QString date = te->date();
            QString flag = te->flag();

            if (directory == 0)
            {
                //kdDebug() << "currentDirectory == 0" << endl;
                return true;
            }
            QString url = '/' + directory->path() + '/' + filename;
            if ( perm[0] == 'd' )
            {
                //get pixmapForURL to return a directory...This is ugly.
                url = '/';
            }
            QPixmap icon = KMimeType::pixmapForURL(url, 0, KIcon::Small);
            //kdDebug() << "listing..." << filename.latin1() << endl;
            //remove "<empty>" from the current directory, if it exists
            kdarListViewItem * firstItem = directory->firstChild();
            if ( firstItem )
            {
                if ( firstItem->text( 0 ) == "<empty>" )
                {
                    delete firstItem;
                    firstItem = 0;
                }
            }
            //kdDebug() << "adding " << ( directory->path() + QString( "/" ) + filename ).latin1() << endl;
            kdarListViewItem *newItem =  new kdarListViewItem( directory, filename, perm, uid, gid, size, date, flag );
            newItem->setPixmap( 0, icon );
            //give this item an "<empty>" child if is a directory
            if ( perm[0] == 'd' )
            {
                (void) new kdarListViewItem( newItem, "<empty>" );
            }
            return true;
        }
        if ( e->type() == (QEvent::Type) KDar::STATUSBAR )  // It must be a kdarStatusBarEvent
        {
            //kdDebug() << "kdar::event: It is a kdarStatusBarEvent...posting message." << endl;
            kdarStatusBarEvent* se = (kdarStatusBarEvent*)e;
            kdar * m_kdar = static_cast<kdar*>(parent());
            if ( m_kdar )
            {
                m_kdar->statusBar()->message( se->message(), se->duration() );
            }
            QString message( se->message() );
            if ( m_kdarconfig->logLevel() == KDar::LOG_FULL  && !message.isEmpty() )
            {
                m_controller->writeLogMessage( message );
            }
            return true;
        }
        if ( e->type() == (QEvent::Type) KDar::SETCURSOR )  // It must be a kdarSetCursorEvent
        {
            //kdDebug() << "kdar::event: It is a kdarSetCursorEvent...changing cursor." << endl;
            kdarSetCursorEvent* sce = (kdarSetCursorEvent*)e;
            setCursor( sce->cursor() );
            return true;
        }
        if ( e->type() == (QEvent::Type) KDar::POSTLISTVIEWITEM )  // A kdarPostListViewItemEvent
        {
            kdDebug() << "kdar::event(): received KDar::POSTLISTVIEWITEM event." << endl;
            kdarPostListViewItemEvent* pe = (kdarPostListViewItemEvent*) e;
            kdDebug() << "name is " << pe->text().latin1() << " and clearView() is " << pe->clearView() << endl;
            if ( pe->clearView() )
            {
                archiveFileView->clear();
            }
            archiveItem = new kdarListViewItem( archiveFileView, pe->text() );
            //set kdar pixmap
            QPixmap smallIcon = SmallIcon("kdar");
            archiveItem->setPixmap(0, smallIcon);
            //This item will contain children:
            (void) new kdarListViewItem( archiveItem, "<empty>" );
            return true;
        }
        if ( e->type() == (QEvent::Type) KDar::CHECKFILESIZE )  // A kdarCheckFileSizeEvent
        {
            kdar * m_kdar = static_cast<kdar*>(parent());
            kdarCheckFileSizeEvent* ce = (kdarCheckFileSizeEvent*) e;
            if ( ce->checkFileSizeAction() == kdarCheckFileSizeEvent::STARTTIMER )
            {
                if ( !m_createConfig->dryRun() )
                {
                    QString mediaText( m_createConfig->mediaText() );
                    if ( m_kdar ) m_kdar->statusBar()->insertItem( "Writing " + mediaText + " slice 1... ", 1, 0, true );
                    //create a QTimer for checking the filesize during
                    //creation operations
                    //This is meant to be a primitive form of feedback
                    //for the user
                    //Hopefully libdar will evolve to allow more detailed
                    //feedback for all of its operations. The timer is started
                    //and stopped in the createThread.
                    fileSizeTimer = new QTimer( this );
                    connect( fileSizeTimer, SIGNAL( timeout() ), this, SLOT( slotCreationCheckFileSize() ) );
                    fileSizeTimer->start( CHECKINTERVAL );
                }
                else
                {
                    if ( m_kdar ) m_kdar->statusBar()->insertItem( "Dry run: not writing slices.", 1, 0, true );
                }
            }
            else
            {
                if ( !m_createConfig->dryRun() )
                {
                    if ( fileSizeTimer )
                    {
                        fileSizeTimer->stop();
                        delete fileSizeTimer;
                        fileSizeTimer = 0;
                    }
                }
                if ( m_kdar ) m_kdar->statusBar()->removeItem( 1 );
            }
            return true;
        }
        if ( e->type() == (QEvent::Type) KDar::PROGRESS )  // It must be a kdarProgressEvent
        {
            kdarProgressEvent* pe = (kdarProgressEvent*)e;
            kdar * m_kdar = static_cast<kdar*>(parent());
            switch ( pe->command() )
            {
                case kdarProgressEvent::PROGRESS_START:
                    if ( m_createConfig->verbose() )
                    {
                        progressBar = new KProgress( this );
                        progressBar->setMaximumWidth( 100 );
                        progressBar->setTotalSteps( 100 );
                        //initialize the file counters
                        m_filesProcessedAdded = 0;
                        m_filesProcessedSkipped = 0;
                        m_filesProcessedTotal = 0;
                        //Show progress bar in statusbar
                        if ( m_kdar ) m_kdar->statusBar()->addWidget( progressBar, 0, true );
                        //A QTimer for progress checking (during creation)
                        progressTimer = new QTimer( this );
                        connect( progressTimer, SIGNAL( timeout() ), this, SLOT( slotUpdateProgressBar() ) );
                        progressTimer->start( PROGRESSINTERVAL );
                    }
                    break;
                case kdarProgressEvent::PROGRESS_STOP:
                    if ( m_createConfig->verbose() )
                    {
                        if ( progressTimer )
                        {
                            progressTimer->stop();
                            delete progressTimer;
                            progressTimer = 0;
                        }
                        if ( progressBar )
                        {
                            if ( m_kdar )
                            {
                                m_kdar->statusBar()->removeWidget( progressBar );
                            }
                            delete progressBar;
                            progressBar = 0;
                        }
                        m_totalFiles = 0;
                    }
                    break;
                case kdarProgressEvent::PROGRESS_INCREASE_ADDED:
                    if ( m_createConfig->verbose() )
                    {
                        ++m_filesProcessedAdded;
                    }
                    break;
                case kdarProgressEvent::PROGRESS_INCREASE_SKIPPED:
                    if ( m_createConfig->verbose() )
                    {
                        ++m_filesProcessedSkipped;
                    }
                    break;
                case kdarProgressEvent::PROGRESS_INCREASE_TOTAL:
                    if ( m_createConfig->verbose() )
                    {
                        ++m_filesProcessedTotal;
                    }
                    break;
                case kdarProgressEvent::PROGRESS_SET_TOTAL:
                    if ( m_createConfig->verbose() )
                    {
                        m_totalFiles = pe->total();
                    }
                    break;
                default:
                    kdDebug() << "kdarView::event(): invalid kdarProgressEvent command." << endl;
            }
            //The progress bar is updated from a QTimer.
            return true;
        }
        if ( e->type() == ( QEvent::Type ) KDar::ARCHIVEBROWSERUPDATES )
        {
            kdarArchiveBrowserUpdatesEvent *ae = (kdarArchiveBrowserUpdatesEvent*) e;
            switch ( ae->command() )
            {
                case kdarArchiveBrowserUpdatesEvent::DISABLE:
                    archiveFileView->setSorting(-1);
                    //archiveFileView->setUpdatesEnabled( false );
                    //std::cout << "kdarcontroller::customEvent(): DISABLE BROWSER" << std::endl;
                    break;
                case kdarArchiveBrowserUpdatesEvent::ENABLE:
                    archiveFileView->setSorting(1);
                    //archiveFileView->setUpdatesEnabled( true );
                    archiveFileView->repaint();
                    //std::cout << "kdarcontroller::customEvent(): ENABLE BROWSER" << std::endl;
                    break;
                default:
                    kdDebug() << "kdarView::event(): invalid kdarArchiveBrowserUpdatesEvent command." << endl;
            }
            return true;
        }
        if ( e->type() == (QEvent::Type) KDar::CANCELBUTTON )
        {
            kdarCancelButtonEvent* cbe = (kdarCancelButtonEvent*)e;
            kdar * m_kdar = static_cast<kdar*>(parent());
            if ( m_kdar )
            {
                switch ( cbe->command() )
                {
                    case kdarCancelButtonEvent::ENABLE:
                        m_kdar->setCancelButtonState( true );
                        break;
                    case kdarCancelButtonEvent::DISABLE:
                        m_kdar->setCancelButtonState( false );
                        break;
                    default:
                        kdDebug() << "kdarView::event(): invalid kdarCancelButtonEvent command." << endl;
                }
            }
        }
    }
    return QWidget::event( e );
}

void kdarView::print(QPainter *p, KPrinter *printer, QPaintDeviceMetrics *metrics)
{
    // do the actual printing, here
    //copied basically from $QTDIR/examples/action/application.cpp
    int margin = 0;
    QRect body( margin, margin, metrics->width()-2*margin, metrics->height()-2*margin );
    QSimpleRichText richText( QStyleSheet::convertFromPlainText (archiveTextBrowser->text()), archiveTextBrowser->font(), archiveTextBrowser->context(), archiveTextBrowser->styleSheet(), archiveTextBrowser->mimeSourceFactory(), body.height() );
    richText.setWidth( p, body.width() );
    QRect view( body );
    int page = 1;
    do
    {
        richText.draw( p, body.left(), body.top(), view, colorGroup() );
        view.moveBy( 0, body.height() );
        p->translate( 0 , -body.height() );
        p->drawText( view.right() - p->fontMetrics().width( QString::number( page ) ), view.bottom() + p->fontMetrics().ascent() + 5, QString::number( page ) );
        if ( view.top()  >= richText.height() )
        {
            break;
        }
        printer->newPage();
        page++;
    } while (TRUE);
}

void kdarView::readProperties()
{
    // the 'config' object points to the session managed
    // config file.  this function is automatically called whenever
    // the app is being restored.  read in here whatever you wrote
    // in 'saveProperties'
    kdDebug() << "kdarView::readProperties(): restoreLayout()" << endl;
    archiveFileView->restoreLayout( m_kdarconfig->globalConfig(), "KDar Layout" );
    //Read in the mainWindowSplitter sizes
    m_kdarconfig->globalConfig()->setGroup( "KDar Layout" );
    QValueList<int> splitterSizes = m_kdarconfig->globalConfig()->readIntListEntry( "mainWindowSplitterSizes" );
    if ( ! splitterSizes.empty() )
    {
        mainWindowSplitter->setSizes( splitterSizes );
    }

    kdDebug() << "kdarView::readProperties(): readGlobalSettings()" << endl;
    m_kdarconfig->readGlobalSettings();

    //Ask user for default storage directory if it doesn't exist yet...
    kdDebug() << "kdarView::readProperties(): storageDir setup" << endl;
    QString storageDir = m_kdarconfig->archiveStorageDirectory();
    QString backupDir = "";
    if ( storageDir[0] != '/' )
    {
        newArchiveStorageDirDialog storageDirDlg;
        bool finished( false );
        QString homeDir = getenv( "HOME" );
        homeDir += "/";
        if ( homeDir.isNull() )
        {
            homeDir = "";
        }
        backupDir = homeDir + "dar_backups/";
        storageDirDlg.newArchiveStorageDir->lineEdit()->setText( backupDir );
        while ( !finished )
        {
            if ( storageDirDlg.exec() )
            {
                storageDir = storageDirDlg.newArchiveStorageDir->lineEdit()->text();
                if ( storageDir[0] == '/' )
                {
                    finished = true;
                }
            }
            else
            {
	        //The user can now skip this step. The user can change this
                //later to a different directory. Will default to $HOME.
                storageDir = homeDir;
                finished = true;
            }
        }
        QDir defaultStorageDir( storageDir );
        if ( !defaultStorageDir.exists() )
        {
            //The directory does not exist: ask user whether to create it
            int result = KMessageBox::questionYesNoList( 0, i18n( "The directory that you chose does not exist. Do you want to create it?" ), storageDir, i18n( "Create directory?" ) );
            if ( result == KMessageBox::Yes )
            {
                //Create the directory.
                if ( !defaultStorageDir.mkdir( storageDir ) )
                {
                    KMessageBox::sorry( 0, i18n( "Unable to create backup directory. Please choose a new backup directory in the \"Settings\" dialog." ) );
                    storageDir = homeDir;
                }
            }
            else
            {
                storageDir = homeDir;
            }
        }
        m_kdarconfig->setArchiveStorageDirectory( storageDir );
        m_createConfig->setArchiveStorageDirectory( storageDir );
        m_restoreConfig->setArchiveStorageDirectory( storageDir );
    }
    kdDebug() << "kdarView::readProperties(): m_kdar->restoreRecentURLEntries()" << endl;
    kdar * m_kdar = static_cast<kdar*>(parent());
    if ( m_kdar ) m_kdar->restoreRecentURLEntries( m_kdarconfig->globalConfig() );
    kdDebug() << "kdarView::readProperties(): m_kdarconfig->updateGUI()" << endl;
    m_kdarconfig->updateGUI();
    kdDebug() << "kdarView::readProperties(): updateGUI()" << endl;
    updateGUI();

}

void kdarView::saveProperties()
{
    // the 'config' object points to the session managed
    // config file.  anything you write here will be available
    // later when this app is restored
        kdDebug() << "setting config file params" << endl;

    archiveFileView->saveLayout(m_kdarconfig->globalConfig(), "KDar Layout");
    //Save the mainWindowSplitter sizes:
    m_kdarconfig->globalConfig()->setGroup( "KDar Layout" );
    m_kdarconfig->globalConfig()->writeEntry( "mainWindowSplitterSizes", mainWindowSplitter->sizes() );

    kdDebug() << "saving the kdarconfig settings" << endl;
    m_kdarconfig->writeGlobalSettings();

    kdar * m_kdar = static_cast<kdar*>( parent() );
    if ( m_kdar )
    {
        m_kdar->saveRecentURLEntries( m_kdarconfig->globalConfig() );
    }
}

void kdarView::slotArchiveChoose( const QString & name )
{
    QString archiveName( name );
    //If archiveName was not passed, user chooses an archive file to open
    //Check that the archiveName file exists. Otherwise, cancel.
    archiveHandle archiveNameHandle( name );
    QFile archiveFile( archiveNameHandle.getArchivePath() + "/" + archiveNameHandle.getArchiveBasename() + ".1." + EXTENSION );
    kdDebug() << "file exists is " << archiveFile.exists() << endl;
    if ( archiveName.isEmpty() || !archiveFile.exists() )
    {
        archiveName = KFileDialog::getOpenFileName(m_kdarconfig->archiveStorageDirectory(), (QString) "*.dar", this, (QString) "Select archive to open");
    }
    //if user cancels, do nothing
    if ( archiveName.isEmpty() )
    {
        kdDebug() << "kdarView::slotArchiveChoose(): user cancelled opening an archive. Returning." << endl;
        return;
    }
    if ( archiveName[0] == '/' )
    {
        m_kdarconfig->setArchiveName( archiveName );
        kdDebug() << "about to open the archive " << archiveName.latin1() << endl;
        try
        {
            kdar * m_kdar = static_cast<kdar*>( parent() );
            m_controller->openArchive( m_kdarconfig );
            if ( m_kdar ) m_kdar->addRecentURL( archiveName );
        }
        catch( libdar::Egeneric &e )
        {
            libdar::user_interaction_warning(e.get_message());
        }
    }
    else
    {
        kdDebug() << "kdarView::slotArchiveChoose(): FIXME to open archives in current directory, from commandline." << endl;
    }
}

void kdarView::slotArchiveChoose( const KURL & name )
{
    //If the file doesn't exist, then remove it from the recent list
    if ( !QFile::exists( name.path() ) )
    {
        //remove from recent
        kdar * m_kdar = static_cast<kdar*>( parent() );
        if ( m_kdar ) m_kdar->removeRecentURL( name );
    }
    else
    {
        slotArchiveChoose( name.path() );
    }
}

void kdarView::slotArchiveContextMenu( KListView *, QListViewItem* contextItem, const QPoint & point )
{
    if (contextItem != 0)
    {
        kdarListViewItem * kdarContextItem = static_cast<kdarListViewItem*>(contextItem);
        kdDebug() << "Context menu item: " << kdarContextItem->text(0).latin1() << endl;
        kdar * m_kdar = static_cast<kdar*>( parent() );
        if ( m_kdar )
        {
            m_kdar->popupContextMenu( point );
        }
    }
}

void kdarView::slotArchiveCreate()
{
    //create an archive of filesystem data

    //This code prepares the m_createConfig object for the controller.
    bool attemptRun(false);
    if ( m_kdarconfig->useCreateArchiveWizard() )
    {
        attemptRun = createDlgWizard->exec();
    }
    else
    {
        attemptRun = createDlg->exec();
    }
    //Put the creation code in a loop, so user can fix the settings if there
    //was a problem (i.e. the storage directory is the root directory)
    //TODO: This is probably overkill. Must be a better way.
    bool goodToGo( true );
    while ( attemptRun )
    {
        //Add the archive name, directory to backup, reference archive,
        //and dry run settings to the m_createConfig object.
        QString newArchiveDirectory;
        QString archiveBaseTest;
        if ( m_kdarconfig->useCreateArchiveWizard() )
        {
            //get the root directory (the one to backup)
            newArchiveDirectory=createDlgWizard->rootDirectoryKURLRequester->lineEdit()->text();
            //if user cancels, do nothing
            if ( newArchiveDirectory.isEmpty() )
            {
                KMessageBox::sorry(this, i18n( "create archive: sorry", "You did not select a directory to backup." ) );
                goodToGo = false;
            }
            //do a check that the directory exists, eh!
            //if !fileexists... return;
            //get the  new archive basename
            archiveBaseTest = createDlgWizard->archiveNameKURLRequester->lineEdit()->text();
            if ( goodToGo )
            {
                if ( archiveBaseTest.isEmpty() || ( archiveBaseTest.findRev( '/' ) == ( (int ) archiveBaseTest.length() -1 ) ) )
                {
                    KMessageBox::sorry(this, i18n( "create archive: sorry", "You did not choose a name for the archive." ) );
                    goodToGo = false;
                }
            }
            if ( goodToGo )
            {
                //Insert full path into archiveBaseTest if necessary.
                if ( archiveBaseTest[0] != '/' )
                {
                    //User wants to use the default storage directory.
                    archiveBaseTest = m_createConfig->archiveStorageDirectory() + archiveBaseTest;
                }
                //Check that the storage directory and the directory to
                //back up are not the same
                KURL storageNameKURL( archiveBaseTest );
                KURL backupNameKURL( newArchiveDirectory );
                kdDebug() << "storage directory: " << storageNameKURL.directory().latin1() << endl;
                kdDebug() << "backup directory: " << backupNameKURL.directory().latin1() << endl;
                if ( storageNameKURL.directory() == backupNameKURL.path( -1 ) )
                {
                    kdDebug() << "the directories are the same!" << endl;
                    if ( m_createConfig->libdarCreateFileMask().is_covered( ( storageNameKURL.fileName() + QString( ".1." ) + QString ( EXTENSION ) ).latin1() ) )
                    {
                        //Display a message
                        KMessageBox::sorry( this, i18n( "create archive: sorry", "Your archive would be created in the root directory. If you really want to do this, you must exclude the archive from inclusion using a file filter. Otherwise, create the archive in another directory." ) );
                        goodToGo = false;
                    }
                }
            }
            if ( goodToGo )
            {
                //Do a check here to make sure that all of the directory
                //filters are subdirectories of the root

                //The root directory
                kdDebug() << "kdarView::slotArchiveCreate(): root is " << newArchiveDirectory << endl;
                QStringList directoryList = m_createConfig->directoryMaskList();
                QString badDirectoryFilters( "" );
                QString cleanRoot = KDarConfig::cleanPath( newArchiveDirectory );
                uint len = cleanRoot.length();

                for( QStringList::Iterator directoryIterator = directoryList.begin(); directoryIterator != directoryList.end(); ++directoryIterator )
                {
                    if ( KDarConfig::cleanPath( *directoryIterator ).left( len ) != cleanRoot )
                    {
                        badDirectoryFilters += KDarConfig::cleanPath( *directoryIterator )+ "\n";
                        goodToGo = false;
                    }
                }
                if ( !goodToGo )
                {
                    //Display message:
                    KMessageBox::sorry( this, i18n( "The following directory filters are not subdirectories of \"%1\":\n %2 Please remove them from the list in the configuration dialog." ).arg( cleanRoot).arg( badDirectoryFilters ) );
                }
            }
            if ( goodToGo && createDlgWizard->diffBackupCheckBox->isChecked() )
            {
                if ( createDlgWizard->referenceArchiveKURLRequester->lineEdit()->text().isEmpty() )
                {
                    KMessageBox::sorry(this, i18n( "create archive: sorry", "You did not choose a reference archive." ) );
                    goodToGo = false;
                }
            }
        } // end of wizard code
        else
        {
            //user not using the wizard. Here is the advanced dialog code.
            //get the root directory (the one to backup)
            newArchiveDirectory = createDlg->rootDirectoryKURLRequester->lineEdit()->text();
            //if user cancels, do nothing
            if ( newArchiveDirectory.isEmpty() )
            {
                KMessageBox::sorry(this, i18n( "create archive: sorry", "You did not select a directory to backup." ) );
                goodToGo = false;
            }
            //do a check that the directory exists, eh!
            //if !fileexists... return;
            //get the  new archive basename
            archiveBaseTest = createDlg->archiveNameKURLRequester->lineEdit()->text();
            if ( goodToGo )
            {
                if ( archiveBaseTest.isEmpty() || ( archiveBaseTest.findRev( '/' ) == ( (int ) archiveBaseTest.length() -1 ) ) )
                {
                    KMessageBox::sorry(this, i18n( "create archive: sorry", "You did not choose a name for the archive." ) );
                    goodToGo = false;
                }
            }
            if ( goodToGo )
            {
                //Insert full path into archiveBaseTest if necessary.
                if (archiveBaseTest[0] != '/')
                {
                    //User wants to use the default storage directory.
                    archiveBaseTest = m_createConfig->archiveStorageDirectory() + archiveBaseTest;
                }
                //Check that the storage directory and the directory to back up are not the same
                KURL storageNameKURL( archiveBaseTest );
                KURL backupNameKURL( newArchiveDirectory );
                kdDebug() << "storage directory: " << storageNameKURL.directory().latin1() << endl;
                kdDebug() << "backup directory: " << backupNameKURL.directory().latin1() << endl;
                if ( storageNameKURL.directory() == backupNameKURL.path( -1 ) )
                {
                    kdDebug() << "the directories are the same!" << endl;
                    if ( m_createConfig->libdarCreateFileMask().is_covered( ( storageNameKURL.fileName() + QString( ".1." ) + QString ( EXTENSION ) ).latin1() ) )
                    {
                        //Display a message
                        KMessageBox::sorry( this, i18n( "create archive: sorry", "Your archive would be created in the root directory. If you really want to do this, you must exclude the archive from inclusion using a file filter. Otherwise, create the archive in another directory." ) );
                        goodToGo = false;
                    }
                }
            }
            if ( goodToGo )
            {
                //Do a check here to make sure that all of the directory
                //filters are subdirectories of the root

                //The root directory
                kdDebug() << "kdarView::slotArchiveCreate(): root is " << newArchiveDirectory.utf8() << endl;
                QStringList directoryList = m_createConfig->directoryMaskList();
                QString badDirectoryFilters( "" );
                QString cleanRoot = KDarConfig::cleanPath( newArchiveDirectory );
                uint len = cleanRoot.length();
                for( QStringList::Iterator directoryIterator = directoryList.begin(); directoryIterator != directoryList.end(); ++directoryIterator )
                {
                    if ( KDarConfig::cleanPath( *directoryIterator ).left( len ) != cleanRoot )
                    {
                        badDirectoryFilters += KDarConfig::cleanPath( *directoryIterator )+ "\n";
                        goodToGo = false;
                    }
                }
                if ( !goodToGo )
                {
                    //Display message:
                    KMessageBox::sorry( this, i18n( "The following directory filters are not subdirectories of \"%1\":\n %2 Please remove them from the list in the configuration dialog." ).arg( cleanRoot ).arg( badDirectoryFilters ) );
                }
            }
            if ( goodToGo && createDlg->diffBackupCheckBox->isChecked() )
            {
                if ( createDlg->referenceArchiveKURLRequester->lineEdit()->text().isEmpty() )
                {
                    KMessageBox::sorry(this, "You did not choose a reference archive.");
                    goodToGo = false;
                }
            }
        }
        //Start the creation thread:
        if ( goodToGo )
        {
            //Set attemptRun to false so we don't execute this again!
            attemptRun = false;
            //Set the createConfig's newArchiveDirectory:
            m_createConfig->setDirectoryToBackup( newArchiveDirectory );
            //Set the NewArchiveName
            m_createConfig->setNewArchiveName( archiveBaseTest );
            //Set the differentialBackup status
            if ( m_kdarconfig->useCreateArchiveWizard() )
            {
                m_createConfig->setDifferentialBackup( createDlgWizard->diffBackupCheckBox->isChecked() );
                //Set the differentialBackupArchiveName
                //Prepend with default directory if needed:
                //Insert full path into archiveBaseTest if necessary.
                QString refArchive = createDlgWizard->referenceArchiveKURLRequester->lineEdit()->text();
                if ( refArchive[0] != '/' )
                {
                    //User wants to use the default storage directory.
                    refArchive = m_createConfig->archiveStorageDirectory() + refArchive;
                }
                m_createConfig->setDifferentialBackupArchiveName( refArchive );
                //Set the dry run option
                m_createConfig->setDryRun( createDlgWizard->dryRunCheckBox->isChecked() );
            }
            else
            {
                m_createConfig->setDifferentialBackup( createDlg->diffBackupCheckBox->isChecked() );
                //Set the differentialBackupArchiveName
                //Prepend with default directory if needed:
                //Insert full path into archiveBaseTest if necessary.
                QString refArchive = createDlg->referenceArchiveKURLRequester->lineEdit()->text();
                if ( refArchive[0] != '/' )
                {
                    //User wants to use the default storage directory.
                    refArchive = m_createConfig->archiveStorageDirectory() + refArchive;
                }
                m_createConfig->setDifferentialBackupArchiveName( refArchive );
                //Set the dry run option
                m_createConfig->setDryRun( createDlg->dryRunCheckBox->isChecked() );
            }
            if ( m_createConfig->differentialBackup() )
            {
                //Set the archiveName in m_createConfig
                m_createConfig->setArchiveName( m_createConfig->differentialBackupArchiveName() );
            }

            //Calculate the total number of files that will be added:
            //Should be able to do this on a time comparable to unix
            //command "du -s"
            if ( m_createConfig->verbose() )
            {
                m_tmpTotalFiles = 0;
                m_rootDirListJob = KIO::listRecursive( m_createConfig->directoryToBackup(), false );
                connect( m_rootDirListJob, SIGNAL( entries( KIO::Job *, const KIO::UDSEntryList & ) ), this, SLOT( slotIncrementTotalFiles( KIO::Job *, const KIO::UDSEntryList & ) ) );
                connect( m_rootDirListJob, SIGNAL( result( KIO::Job * ) ), this, SLOT( slotSetTotalFiles( KIO::Job * ) ) );
            }
            //Ask the controller to start the creation process.
            m_controller->createArchive( m_createConfig );
            //break out of the while loop.
        }
        else
        {
            //There was a problem with the creation settings or
            //the user cancelled, so try again.
            if ( m_kdarconfig->useCreateArchiveWizard() )
            {
                attemptRun = createDlgWizard->exec();
            }
            else
            {
                attemptRun = createDlg->exec();
            }
            goodToGo = true;
        }
    }
}

void kdarView::slotArchiveDescend(QListViewItem * expandItem)
{
    //populate an item in the KListView with the contents of an archive
    //subdirectory
    m_controller->populateArchiveTree( 0, expandItem );
}

void kdarView::slotArchiveDiff()
{
    //compare the archive against the filesystem
    m_controller->diffArchive( m_kdarconfig );
}

void kdarView::slotArchiveExtract()
{
    //restore an archive onto the filesystem
    if ( !theArchive )
    {
        slotArchiveChoose();
        m_controller->waitForThread( controller::OPEN_THREAD );
    }
    if ( !theArchive )
    {
        KMessageBox::sorry( this, i18n( "restore archive: sorry", "Could not open the archive for restoring." ) );
        return;
    }
    if ( extractDlg->exec() )
    {
        QString extractArchiveDirectory = extractDlg->extractKURLRequester->lineEdit()->text();
        //if user cancels, do nothing
        kdDebug() << "slotArchiveRestore(): extract directory is " << extractArchiveDirectory.latin1() << endl;
        if ( extractArchiveDirectory.isEmpty() || extractArchiveDirectory[0] != '/' )
        {
            //Inform user that they need to specify a directory
            KMessageBox::sorry( this, i18n( "restore archive: sorry", "You did not select a valid restoration directory." ) );
            return;
        }
        m_restoreConfig->setExtractArchiveDirectory( extractArchiveDirectory );
        m_restoreConfig->setDryRun( extractDlg->dryRunCheckBox->isChecked() );
        m_controller->restoreArchive( m_restoreConfig );
    }
}

void kdarView::slotArchiveIsolate()
{
    m_controller->isolateArchive( m_kdarconfig );
}

void kdarView::slotArchiveList( const bool loadArchive )
{
    //Initiate an archive listing
    m_controller->listArchive( m_kdarconfig, loadArchive );
}

void kdarView::slotArchiveTest()
{
    //test the archive
    m_controller->testArchive( m_kdarconfig );
}

void kdarView::slotClearMessages()
{
    //Clear the message window
    archiveTextBrowser->clear();
}

void kdarView::slotCreationCheckFileSize()
{
    //Stop the timer
    if ( fileSizeTimer ) fileSizeTimer->stop();
    //Check the filesize
    archiveHandle newArchiveHandle( m_createConfig->newArchiveName() );
    long i = newArchiveHandle.lastArchiveSliceNumber();
    QFile newArchiveFile( m_createConfig->newArchiveName() + QString( "." ) + QString::number( i ) + QString( "." ) + EXTENSION );
    if ( newArchiveFile.exists() )
    {
        long sizeKB( newArchiveFile.size()/1024 );
        long sizeMB( newArchiveFile.size()/1048576 );
        long size;
        QString sizeSuffix;
        if ( sizeKB < 2048 )
        {
            size = sizeKB;
            sizeSuffix = i18n( "archive current slice size, kilobytes", "kB" );
        }
        else
        {
            size = sizeMB;
            sizeSuffix = i18n( "archive current slice size, megabytes", "MB" );
        }
        QString mediaText = m_createConfig->mediaText();
        QString statusText;
        statusText = QString( i18n( "e.g., Writing DVD (4.7GB) slice 2...3534 MB", "Writing %1 slice %2...%3 %4" ) ).arg( mediaText ).arg( QString::number( i ) ).arg( size, 4 ).arg( sizeSuffix );
        kdar * m_kdar = static_cast<kdar*>(parent());
        if ( m_kdar ) m_kdar->statusBar()->changeItem( statusText, 1 );
    }
    else
    {
        kdDebug() << "kdar::slotCreationCheckFileSize(): " << m_createConfig->newArchiveName().latin1() << "does not exist." << endl;
    }
    //Start timer again
    if ( fileSizeTimer ) fileSizeTimer->start( CHECKINTERVAL );
}

void kdarView::slotUpdateProgressBar()
{
    //Update the progress bar with current number of files added to archive.
    if ( m_createConfig->verbose() )
    {
        if ( m_totalFiles > 0 )
        {
            int percentage( (int) ( (float) m_filesProcessedAdded/ (float) m_totalFiles * 100) );
            //kdDebug() << "percentage is " << percentage << endl;
            if ( progressBar ) progressBar->setProgress( percentage );
        }
    }
}

void kdarView::slotIncrementTotalFiles( KIO::Job *, const KIO::UDSEntryList & UDSEntries )
{
    //Loop through all items:
    //If the item is covered by the mask, the file will be added to the archive.
    //iterator
    KIO::UDSEntryListConstIterator it = UDSEntries.begin();
    KIO::UDSEntryListConstIterator end = UDSEntries.end();

    for ( ; it != end; ++it )
    {
        try
        {
            //is the file covered by file and directory masks?
            KFileItem fileItem( *it, ""/*KFileItem::Unknown?*/, false, true );
            //If a directory filter excludes a directory, all files below
            //must be excluded.
            KURL fileURL( m_createConfig->directoryToBackup() + fileItem.text() );
            //kdDebug() << "kdar::slotIncrementTotalFiles(): fileURL is " << fileURL.path().latin1() << endl;
            KURL baseURL( m_createConfig->directoryToBackup() );
            bool dirExcluded( false );
            libdar::path filePath( fileURL.path( -1 ).latin1() );
            std::string s;
            do
            {
                dirExcluded = dirExcluded || !m_createConfig->libdarCreateDirectoryMask().is_covered( filePath.display() );
                //kdDebug() << "filePath: " << filePath.display().c_str() << ": dirExcluded " << dirExcluded <<endl;
            } while ( filePath.pop( s ) && !dirExcluded && ( QString( filePath.display().c_str() ) != baseURL.path( -1 ) ) );
            libdar::path filename( fileItem.text().latin1() );
            bool fileCovered = m_createConfig->libdarCreateFileMask().is_covered( filename.basename() );
            //kdDebug() << "kdar::slotIncrementTotalFiles(): fileDirectory is " << fileDirectory.latin1() << endl;
            if ( !dirExcluded && ( fileItem.isDir() || fileCovered ) )
            {
                //increase the file count: using a temporary counter means the
                //total file count remains 0, and the progress indicator isn't
                //updated, until the KDirLister is finished.
//              kdDebug() << "kdar::slotIncrementTotalFiles(): increasing tmpTotalFiles: " << filename.latin1() << endl;
                ++m_tmpTotalFiles;
                //kdDebug() << fileItem.text().latin1() << ": dirExcluded: " << dirExcluded << ", fileCovered: " << fileCovered << ", isDir(): " << fileItem.isDir() << endl;
            }
            else
            {
                if ( fileItem.isDir() && m_createConfig->keepPruned() )
                {
                        ++m_tmpTotalFiles;
                }
                //kdDebug() << fileItem.text().latin1() << ": dirExcluded: " << dirExcluded << ", fileCovered: " << fileCovered << ", isDir(): " << fileItem.isDir() << ", skipping" << endl;
            }
        }
        catch( libdar::Egeneric &e )
        {
            kdDebug() << "Exception caught: kdar::slotIncrementTotalFiles()" << endl;
        }
    }
}

void kdarView::slotSetTotalFiles( KIO::Job* )
{
    if ( m_createConfig->verbose() )
    {
        //This is because we don't count the "." and ".." from
        //the listRecursive job.
        if ( m_tmpTotalFiles > 2 ) m_tmpTotalFiles -= 2;
        //I don't know why the totalFiles ends up being 1 more than KDar stores.
        if ( m_tmpTotalFiles > 1 ) m_tmpTotalFiles -= 1;
        //kdDebug() << "kdar::slotSetTotalFiles(): listRecursive job finished. m_totalFiles is " << m_tmpTotalFiles << endl;
        kdarProgressEvent *progressSetTotalFiles = new kdarProgressEvent( kdarProgressEvent::PROGRESS_SET_TOTAL, m_tmpTotalFiles );
        QApplication::postEvent( this, progressSetTotalFiles );
        //Delete the listJob
        //Is the listjob deleted automatically?
        //if ( m_rootDirListJob )
        //{
        //    delete m_rootDirListJob;
        //    m_rootDirListJob = 0;
        //}
    }
}

void kdarView::slotConfigureCreationSettings()
{
    //Copy the createConfig instance so that we can
    //cancel the changes if necessary
    std::auto_ptr<KDarConfig> tmpConfig( new KDarConfig(this, 0, m_createConfig, false) );
    tmpConfig->setupCreateConfig();
    if ( tmpConfig->exec() )
    {
        *m_createConfig = *tmpConfig;
    }
}

void kdarView::slotConfigureRestoreSettings()
{
    //Copy the restoreConfig instance so that we can cancel the changes if necessary
    std::auto_ptr<KDarConfig> tmpConfig( new KDarConfig(this, 0, m_restoreConfig, false) );
    tmpConfig->setupRestoreConfig();
    if ( tmpConfig->exec() )
    {
        *m_restoreConfig = *tmpConfig;
    }
}

void kdarView::slotSaveCreationProfile()
{
    //Pop up a save file dialog
    QString saveFilename = KFileDialog::getSaveFileName( m_kdarconfig->archiveStorageDirectory(), QString::null, 0, i18n( "saving a creation profile", "Choose a name for the profile" ) );
    //If successful, set the m_createConfig's KConfig to store to the new file.
    kdar *m_kdar = static_cast<kdar*>(parent());
    if ( !saveFilename.isEmpty() )
    {
        KConfig * newConfig = new KConfig( saveFilename, false, false );
        m_createConfig->setGlobalConfig( newConfig );
        if ( m_kdarconfig->useCreateArchiveWizard() )
        {
            m_createConfig->setDirectoryToBackup( createDlgWizard->rootDirectoryKURLRequester->lineEdit()->text() );
            m_createConfig->setNewArchiveName( createDlgWizard->archiveNameKURLRequester->lineEdit()->text() );
            m_createConfig->setDifferentialBackup( createDlgWizard->diffBackupCheckBox->isChecked() );
            m_createConfig->setDifferentialBackupArchiveName( createDlgWizard->referenceArchiveKURLRequester->lineEdit()->text() );
            m_createConfig->setDryRun( createDlgWizard->dryRunCheckBox->isChecked() );
        }
        else
        {
            m_createConfig->setDirectoryToBackup( createDlg->rootDirectoryKURLRequester->lineEdit()->text() );
            m_createConfig->setNewArchiveName( createDlg->archiveNameKURLRequester->lineEdit()->text() );
            m_createConfig->setDifferentialBackup( createDlg->diffBackupCheckBox->isChecked() );
            m_createConfig->setDifferentialBackupArchiveName( createDlg->referenceArchiveKURLRequester->lineEdit()->text() );
            m_createConfig->setDryRun( createDlg->dryRunCheckBox->isChecked() );
        }
        m_createConfig->writeGlobalSettings();
        kdDebug() << "slotSaveCreationProfile(): saved settings." << endl;
        //Display a message that the settings were saved.
        if ( m_kdar ) m_kdar->statusBar()->message( i18n( "saving a creation profile", "Settings saved." ), DURATION );
    }
    else
    {
        //Display a message that the settings were not saved.
        if ( m_kdar ) m_kdar->statusBar()->message( i18n( "saving a creation profile", "Settings not saved." ), DURATION );
    }
}

void kdarView::slotRestoreCreationProfile( const QString & profile )
{
    //Pop up an open file dialog --- the file must exist on disk!
    QString restoreFilename( profile );
    if ( restoreFilename.isEmpty() )
    {
        restoreFilename = KFileDialog::getOpenFileName( m_kdarconfig->archiveStorageDirectory(), QString::null, 0, i18n( "restoring a creation profile", "Choose the profile to restore" ) );
    }
    //If successful, set the m_createConfig's KConfig to restore from the new file.
    kdar *m_kdar = static_cast<kdar*>(parent());
    if ( !restoreFilename.isEmpty() )
    {
        KConfig* newConfig = new KConfig( restoreFilename, false, false );
        m_createConfig->setGlobalConfig( newConfig );
        m_createConfig->readGlobalSettings();
        m_createConfig->updateGUI();
        //update the create dialogs' settings.
        if ( m_kdarconfig->useCreateArchiveWizard() )
        {
            createDlgWizard->rootDirectoryKURLRequester->lineEdit()->setText( m_createConfig->directoryToBackup() );
            if ( !createDlgWizard->rootDirectoryKURLRequester->lineEdit()->text().isEmpty() )
            {
                //The archive root directory page: must be selected to continue...
                createDlgWizard->setNextEnabled( createDlgWizard->page( 2 ), true );
            }
            else
            {
                createDlgWizard->setNextEnabled( createDlgWizard->page( 2 ), false );
            }
            createDlgWizard->archiveNameKURLRequester->lineEdit()->setText( m_createConfig->newArchiveName() );
            if ( !createDlgWizard->rootDirectoryKURLRequester->lineEdit()->text().isEmpty() )
            {
                //This is the archive name page: name must beentered to continue...
                createDlgWizard->setNextEnabled( createDlgWizard->page( 1 ), true );
            }
            else
            {
                createDlgWizard->setNextEnabled( createDlgWizard->page( 1 ), false );
            }
            createDlgWizard->setNextEnabled( createDlgWizard->page( 0 ), true );
            createDlgWizard->diffBackupCheckBox->setChecked( m_createConfig->differentialBackup() );
            createDlgWizard->referenceArchiveKURLRequester->lineEdit()->setText( m_createConfig->differentialBackupArchiveName() );
            createDlgWizard->dryRunCheckBox->setChecked( m_createConfig->dryRun() );
        }
        else
        {
            createDlg->rootDirectoryKURLRequester->lineEdit()->setText( m_createConfig->directoryToBackup() );
            createDlg->archiveNameKURLRequester->lineEdit()->setText( m_createConfig->newArchiveName() );
            createDlg->diffBackupCheckBox->setChecked( m_createConfig->differentialBackup() );
            createDlg->referenceArchiveKURLRequester->lineEdit()->setText( m_createConfig->differentialBackupArchiveName() );
            createDlg->dryRunCheckBox->setChecked( m_createConfig->dryRun() );
        }
        kdDebug() << "slotRestoreCreationProfile(): saved restored." << endl;
        //Display a message that the settings were restored.
        if ( m_kdar ) m_kdar->statusBar()->message( i18n( "restoring a creation profile", "Settings restored." ), DURATION );
    }
    else
    {
        //Display a message that the settings were not restored.
        if ( m_kdar ) m_kdar->statusBar()->message( i18n( "restoring a creation profile", "Settings not restored." ), DURATION );
    }
}

void kdarView::slotSaveRestoreProfile()
{
    //Pop up a save file dialog
    QString saveFilename = KFileDialog::getSaveFileName( m_kdarconfig->archiveStorageDirectory(), QString::null, 0, i18n( "saving a restoration profile", "Choose a name for the profile" ) );
    //If successful, set the m_restoreConfig's KConfig to store to the new file.
    kdar *m_kdar = static_cast<kdar*>(parent());
    if ( !saveFilename.isEmpty() )
    {
        KConfig * newConfig = new KConfig( saveFilename, false, false );
        m_restoreConfig->setGlobalConfig( newConfig );
        m_restoreConfig->setExtractArchiveDirectory( extractDlg->extractKURLRequester->lineEdit()->text() );
        kdDebug() << "kdarView::slotSaveRestoreProfile(): extraction directory is " << m_restoreConfig->extractArchiveDirectory() << endl;
        m_restoreConfig->setDryRun( extractDlg->dryRunCheckBox->isChecked() );
        m_restoreConfig->writeGlobalSettings();
        kdDebug() << "slotSaveRestoreProfile(): saved settings." << endl;
        //Display a message that the settings were saved.
        if ( m_kdar ) m_kdar->statusBar()->message( i18n( "saving a restoration profile", "Settings saved." ), DURATION );
    }
    else
    {
        //Display a message that the settings were not saved.
        if ( m_kdar ) m_kdar->statusBar()->message( i18n( "saving a restoration profile", "Settings not saved." ), DURATION );
    }
}

void kdarView::slotRestoreRestoreProfile()
{
    //Pop up an open file dialog --- the file must exist on disk!
    QString restoreFilename = KFileDialog::getOpenFileName( m_kdarconfig->archiveStorageDirectory(), QString::null, 0, i18n( "restoring a restoration profile", "Choose the profile to restore" ) );
    //If successful, set the m_restoreConfig's KConfig to restore from the new file.
    kdar *m_kdar = static_cast<kdar*>(parent());
    if ( !restoreFilename.isEmpty() )
    {
        KConfig * newConfig = new KConfig( restoreFilename, false, false );
        m_restoreConfig->setGlobalConfig( newConfig );
        m_restoreConfig->readGlobalSettings();
        m_restoreConfig->updateGUI();
        //update the extractDlg's settings.
        kdDebug() << "kdarView::slotRestoreRestoreProfile(): extraction directory is " << m_restoreConfig->extractArchiveDirectory() << endl;
        extractDlg->extractKURLRequester->lineEdit()->setText( m_restoreConfig->extractArchiveDirectory() );
        extractDlg->dryRunCheckBox->setChecked( m_restoreConfig->dryRun() );
        kdDebug() << "kdarView::slotRestoreRestoreProfile(): restored settings." << endl;
        //Display a message that the settings were restored.
        if ( m_kdar ) m_kdar->statusBar()->message( i18n( "restoring a restoration profile", "Settings restored." ), DURATION );
    }
    else
    {
        //Display a message that the settings were not restored.
        if ( m_kdar ) m_kdar->statusBar()->message( i18n( "restoring a restoration profile", "Settings not restored." ), DURATION );
    }
}

void kdarView::slotExportDarCreateCommand()
{
    //Set the new archive name and root directory in the config object
    if ( m_kdarconfig->useCreateArchiveWizard() )
    {
        m_createConfig->setNewArchiveName( createDlgWizard->archiveNameKURLRequester->lineEdit()->text() );
        m_createConfig->setDirectoryToBackup( createDlgWizard->rootDirectoryKURLRequester->lineEdit()->text() );
        m_createConfig->setDifferentialBackup( createDlgWizard->diffBackupCheckBox->isChecked() );
        m_createConfig->setDifferentialBackupArchiveName( createDlgWizard->referenceArchiveKURLRequester->lineEdit()->text() );
    }
    else
    {
        m_createConfig->setNewArchiveName( createDlg->archiveNameKURLRequester->lineEdit()->text() );
        m_createConfig->setDirectoryToBackup( createDlg->rootDirectoryKURLRequester->lineEdit()->text() );
        m_createConfig->setDifferentialBackup( createDlg->diffBackupCheckBox->isChecked() );
        m_createConfig->setDifferentialBackupArchiveName( createDlg->referenceArchiveKURLRequester->lineEdit()->text() );
    }

    //Ask user for filename to export the dar command as
    QString darFilename = KFileDialog::getSaveFileName( m_kdarconfig->archiveStorageDirectory(), QString( "*.sh" ), 0, i18n( "Select a filename for the archive creation command" ) );
    if ( !darFilename.isEmpty() )
    {
        //Open the filename for writing, creating it.
        QFile darFile( darFilename );
        if ( darFile.open( IO_WriteOnly ) )
        {
            QTextStream darStream( &darFile );
            //Use "/bin/bash" for the shell
            darStream << "#!/bin/bash" << endl;
            darStream << endl;
            darStream << i18n( "# Run this script to create an archive using the following dar command." ) << endl;
            darStream << endl;
            //send the create dar command string to the file
            darStream << m_createConfig->buildDarCommandLine( KDar::LIBDAR_CREATE );
            //Change permissions to allow execution of the script
            //Is this a security hole?
            //Wait until qt4 to do this?
            //Close the file.
            darFile.close();
        }
        else
        {
            KMessageBox::sorry( this, i18n( "Failed to open file for export." ) );
        }
    }
}

void kdarView::slotExportDarRestoreCommand()
{
    //Ask user for filename to export the dar command as
    QString darFilename = KFileDialog::getSaveFileName( m_kdarconfig->archiveStorageDirectory(), QString( "*.sh" ), 0, i18n( "Select a filename for the archive restoration command" ) );
    if ( !darFilename.isEmpty() )
    {
        //Open the filename for writing, creating it.
        QFile darFile( darFilename );
        if ( darFile.open( IO_WriteOnly ) )
        {
            QTextStream darStream( &darFile );
            //Use "/bin/bash" for the shell
            darStream << "#!/bin/bash" << endl;
            darStream << endl;
            darStream << i18n( "# Run this script, from a directory that contains an archive, to restore that archive using the following dar command." ) << endl;
            darStream << i18n( "# As it is, this command would restore the named archive (-x argument) onto its original location on the system that created it." ) << endl;
            darStream << i18n( "# Modify the command as needed so that dar restores the archive to the desired location (-R argument) on your system." ) << endl;
            darStream << endl;
            //Use the basename of the archive for the -x argument, not a full path
            KDarConfig *darRestoreConfig = new KDarConfig( this, 0, m_restoreConfig, 0 );
            archiveHandle base( m_restoreConfig->archiveName() );
            darRestoreConfig->setArchiveName( base.getArchiveBasename() );
            darRestoreConfig->setArchiveStorageDirectory( "./" );
            darRestoreConfig->setExtractArchiveDirectory( extractDlg->extractKURLRequester->lineEdit()->text() );
            //send the restore dar command string to the file
            darStream << darRestoreConfig->buildDarCommandLine( KDar::LIBDAR_RESTORE );
            //Change permissions to allow execution of the script
            //Is this a security hole?
            //Wait until qt4 to do this?
            //Close the file.
            darFile.close();
            if ( darRestoreConfig ) delete darRestoreConfig;
            darRestoreConfig = 0;
        }
        else
        {
            KMessageBox::sorry( this, i18n( "Failed to open file for export." ) );
        }
    }
}

void kdarView::updateGUI()
{
    //Update KDar's fonts
    kapp->setFont( m_kdarconfig->generalFont() );
    archiveFileView->setFont( m_kdarconfig->fileBrowserFont() );
    archiveTextBrowser->setFont( m_kdarconfig->messageWindowFont() );
    kdar * m_kdar = static_cast<kdar*>(parent());
    if ( m_kdar ) m_kdar->statusBar()->setFont( m_kdarconfig->statusBarFont() );
}

void kdarView::optionsPreferences()
{
    QString oldArchiveStorageDirectory = m_kdarconfig->archiveStorageDirectory();
    QString statusMessage( i18n( "preferences", "Settings not changed." ) );
    std::auto_ptr<KDarConfig> tmpConfig( new KDarConfig( this, 0, m_kdarconfig, false ) );
    if ( tmpConfig->exec() )
    {
        *m_kdarconfig = *tmpConfig;
        //User has changed some global settings here, so update the
        //m_createConfig and m_restoreConfig objects as well.
        *m_createConfig = *tmpConfig;
        *m_restoreConfig = *tmpConfig;
        statusMessage = i18n( "preferences", "Settings changed." );
        QString nullDir = m_kdarconfig->archiveStorageDirectory();
        if ( nullDir[0] != '/' )
        {
            //Send message that the archiveStorage Directory is unchanged.
            statusMessage = i18n( "preferences", "Settings changed. Invalid archive storage directory: left unchanged." );
            //Put the old archiveStorageDir back into the URLRequester.
            m_kdarconfig->setArchiveStorageDirectory( oldArchiveStorageDirectory );
            m_kdarconfig->archiveStorageDirKURLRequester->lineEdit()->setText( oldArchiveStorageDirectory);
        }
        updateGUI();
        m_kdarconfig->writeGlobalSettings();
    }
    //Send status message.
    kdar * m_kdar = static_cast<kdar*>(parent());
    if ( m_kdar ) m_kdar->statusBar()->message(statusMessage, DURATION);
}

KConfig * kdarView::globalConfig() const
{
    kdDebug() << "kdarView::globalConfig(): returning m_kdarconfig->globalConfig()" << endl;
    return KGlobal::config();
}

void kdarView::slotArchiveOpenFromDoubleClick(QListViewItem * item)
{
    kdDebug() << "kdarView::slotArchiveOpenFromDoubleClick reached." << endl;
    if (!item) return;
    if ( !item->parent() && !item->firstChild() )
    {
        //ensure that we have at least the top level of the listView populated
        slotArchiveList();
    }
    else
    {
        //If the item has children and it is closed, open it, and vice-versa
        item->setOpen(!item->isOpen());
    }
}

void kdarView::showLogWindow( bool setVisible )
{
    if ( logDialog )
    {
        if ( setVisible )
        {
            if ( !QFile::exists( m_kdarconfig->logFile() ) )
            {
                //create the log file
                QFile file( m_kdarconfig->logFile() );
                file.open( IO_WriteOnly | IO_Append );
                file.close();
            }
            logPart->openURL( m_kdarconfig->logFile() );
            logDialog->show();
        }
        else
        {
            logDialog->hide();
            logPart->closeURL();
        }
    }
}

#include "kdarview.moc"
