/***************************************************************************
 *   Copyright (C) 2004 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "openArchiveThread.h"

#include "controller.h"
#include "archiveHandle.h"
#include "kdarConfig.h"
#include "kdardeclarations.h"
#include "kdardialog.h"
#include "kdarListViewItem.h"
#include "kdarPostListViewItemEvent.h"
#include "kdarSetCursorEvent.h"
#include "kdarStatusBarEvent.h"

#include <kcursor.h>
#include <kdebug.h>

#include <qapp.h>
#include <qstring.h>

#include <dar/libdar.hpp>
#include <dar/erreurs.hpp>

#include <string>

extern bool libdarWorking;
extern libdar::archive *theArchive;
extern kdarListViewItem *archiveItem;

openArchiveThread::openArchiveThread( controller * aController, KDarConfig * kdarconfig, bool updateListView )
    : KDarThread( aController, kdarconfig ),
      m_updateListView( updateListView )
{}

openArchiveThread::~openArchiveThread()
{
}

void openArchiveThread::run()
{
    libdarWorking = true;
    setWaitCursor();
    enableCancelButton();
    statusMessage( "Opening archive...\n", FOREVER );

    //check whether we have a valid archiveName:
    archiveHandle archiveHndl( m_kdc->archiveName() );
    try
    {
        kdDebug() << "opening the archive" << endl;
        theArchive = new libdar::archive(archiveHndl.getArchiveLibdarPath(), archiveHndl.getArchiveBasename().latin1(), EXTENSION, libdar::crypto_none, m_kdc->passwordString().latin1(), m_kdc->inputPipe().latin1(), m_kdc->outputPipe().latin1(), m_kdc->executeRef().latin1(), m_kdc->verbose() );
        if ( m_updateListView )
        {
            kdarPostListViewItemEvent *postListViewEvent = new kdarPostListViewItemEvent( archiveHndl.getArchiveBasename(), true );
            QApplication::postEvent( m_controller, postListViewEvent );
        }
        kdDebug() << "finished opening archive" << endl;
    }
    catch( libdar::Egeneric &e )
    {
        kdDebug() << "problem opening archive" << endl;
        libdar::user_interaction_warning(e.get_message());
        if ( theArchive ) delete theArchive;
        theArchive = 0;
        kdDebug() << "caught problem opening archive" << endl;
    }
    setNormalCursor();
    disableCancelButton();
    statusMessage( "Opening archive...done.\n" );
    libdarWorking = false;
    deleteMe();
}

