/***************************************************************************
 *   Copyright (C) 2004 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "populateTreeArchiveThread.h"

#include "controller.h"
#include "kdarConfig.h"
#include "kdardialog.h"
#include "kdarArchiveBrowserUpdatesEvent.h"
#include "kdarListViewItem.h"
#include "kdarSetCursorEvent.h"
#include "kdarStatusBarEvent.h"
#include "kdarWarningEvent.h"

#include <kcursor.h>
#include <kdebug.h>
#include <kiconloader.h>
#include <klistview.h>

#include <qapp.h>
#include <qstring.h>
#include <qpixmap.h>

#include <dar/libdar.hpp>
#include <dar/tools.hpp>
#include <dar/erreurs.hpp>

#include <string>

extern bool libdarWorking;
extern kdarListViewItem *currentDirectory;

extern libdar::archive *theArchive;

populateTreeArchiveThread::populateTreeArchiveThread( controller * aController, QListViewItem * directory )
    : KDarThread( aController, 0 ),
      m_directory( directory )
{}

populateTreeArchiveThread::~populateTreeArchiveThread()
{
}

void populateTreeArchiveThread::run()
{
    kdDebug() << "populateTreeArchiveThread::run() reached." << endl;
    if ( !theArchive ) return;
    libdarWorking = true;
    setWaitCursor();
    kdarListViewItem *expandItem = static_cast<kdarListViewItem *>( m_directory );
    kdDebug() << "populateTreeArchiveThread::run(): expandItem's text is " << expandItem->text( 0 ).latin1() << endl;
    QString path = expandItem->path();
    kdDebug() << "populateTreeArchiveThread::run(): expandItem's path is " << expandItem->path().latin1() << endl;
    currentDirectory = expandItem;
    //Disable archive browser updates:
    disableBrowser( expandItem );
    try
    {
        kdDebug() << "populateTreeArchiveThread::run() trying get_children_of" << endl;
        libdar::get_children_of(theArchive, expandItem->path().latin1());
    }
    catch(libdar::Egeneric &e)
    {
        kdDebug() << "populateTreeArchiveThread::run(): exception caught." << endl;
        libdar::user_interaction_warning(e.get_message());
    }
    setNormalCursor();
    //Enable archive browser updates:
    enableBrowser( expandItem );
    libdarWorking = false;
    kdDebug() << "populateTreeArchiveThread::run() deleting..." << endl;
    deleteMe();
}
