/***************************************************************************
 *   Copyright (C) 2004 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "restoreArchiveThread.h"

#include "kdarConfig.h"
#include "kdardeclarations.h"
#include "kdardialog.h"
#include "kdarListViewItem.h"
#include "kdarSetCursorEvent.h"
#include "kdarstatistics.h"
#include "kdarStatusBarEvent.h"
#include "kdarWarningEvent.h"

#include <kcursor.h>
#include <kdebug.h>
#include <klocale.h>

#include <qdatetime.h>
#include <qstring.h>

#include <dar/libdar.hpp>
#include <dar/erreurs.hpp>

#include <string>

extern bool libdarWorking;
extern libdar::archive *theArchive;
extern QString extractArchiveDirectory;

restoreArchiveThread::restoreArchiveThread( controller * aController, KDarConfig * kdarconfig, KListView * archiveFileView )
    : KDarThread ( aController, kdarconfig, archiveFileView )
{}

restoreArchiveThread::~restoreArchiveThread()
{}

void restoreArchiveThread::run()
{
    libdarWorking = true;
    QDateTime startTime = QDateTime::currentDateTime();
    openLogFile();
    setWaitCursor();
    enableCancelButton();
    logMessage( "\n" + i18n( "BEGIN RESTORE: " ) + QDateTime::currentDateTime().toString( "ddd dd MMM yyyy hh:mm:ss" ) + " " + m_kdc->archiveName() + "\n", true );
    logMessage( "command line: " + m_kdc->buildDarCommandLine( KDar::LIBDAR_RESTORE ) + "\n", true );
    statusMessage( "Restoring files...\n", FOREVER );

    libdar::et_mask subtree = m_kdc->libdarCreateDirectoryMask();
    libdar::ou_mask subIncludes = buildDirectoryMaskFromListView( m_kdc->extractArchiveDirectory() );
    if ( subIncludes.size() > 0 ) subtree.add_mask( subIncludes );
    bool destroy( !m_kdc->noDelete() );

    try
    {
        KDarStatistics ks=op_extract(theArchive, m_kdc->extractArchiveDirectory().latin1(), m_kdc->libdarCreateFileMask(), subtree, m_kdc->allowOverwrite(), m_kdc->warnOnOverwrite(), m_kdc->verbose(), destroy, m_kdc->restoreRecent(), m_kdc->systemEA(), m_kdc->userEA(), m_kdc->flatRestore(), m_kdc->ignoreID(), false, (libdar::infinint) m_kdc->hourShiftInt(), m_kdc->dryRun() );
        logMessage( ks.restoreStatistics( (KDar::logLevel) m_kdc->logLevel() ) );
        statusMessage( "Restoring files...done.\n" );
        logMessage( timeDifference( startTime, QDateTime::currentDateTime() ) );
    }
    catch (libdar::Egeneric &e)
    {
        libdar::user_interaction_warning(e.get_message());
        statusMessage( "Error: could not restore archive.\n" );
    }
    setNormalCursor();
    disableCancelButton();
    m_endMessage = "END RESTORE: " + QDateTime::currentDateTime().toString( "ddd dd MMM yyyy hh:mm:ss" ) + " " + m_kdc->archiveName() + "\n";
    libdarWorking = false;
    deleteMe();
}

